// NEW: continous transmission if port #2 = "LOW"

// selectable output channels  
// --------------------
// Port:   #3      #4
// --------------------
// Ch.1:  "HIGH"  "HIGH"
// Ch.2:  "LOW"   "HIGH"
// Ch.3:  "HIGH"  "LOW"  
// Ch,4:  "LOW"   "LOW"

// http://remco.org/index.php/2017/01/16/adf4351-evaluation-board-microwave-beacon/comment-page-1/#comment-20388
// http://remco.org/ADF435x-attiny13a.ino

// Simple VHF/UHF/SHF beacon with an ADF4351 (evaluation board); Clock= 25MHz 
// Original by (c) PA3FYM, Jan 2017
//  
// Please check the ADF4351 datasheet or consult ADF4351 software
// for proper PLL register values.
//
// This code may be used (by you) for non commercial purposes, 
// and/or intentions, and/or actions as long as you refer to me.

// const byte clk =    12; // PB4 ADF4531 clock  
// const byte data =   11; // PB3 ADF4531 data
// const byte le =     10; // PB2 ADF4531 latch enable

#define cwtick 85          // cw tick speed in msec

long int r0, r1, r2, r3, r4, r5;
 
// gewuenschte Morsezeichensequenz als Folge von Nullen und Einsen zusammenstellen
// Punkt= 1; Strich= 111; Zwischenraum= 0; Beispiel fuer Zeichen "D" ( -.. ) = 1110101
// Pause zwischen Zeichen= 000; Pause zwischen Worten= 00000000 

// DJ7OO   00000000 1110101 000 1011101110111 000 1110111010101 000 11101110111 000 11101110111 00000000
// JN49CX  01011101110111 000 11101 000 10101010111 000 11101110111011101 000 11101011101 000 11101010111 0000     

// jetzt als Bitfolge zusammenfassen (Leerstellen entfernen) und in Bloecke zu je 8 Einzelbits aufteilen

byte message[22]= {0b00000000,0b11101010,0b00101110,0b11101110,0b00111011,0b10101010,0b01110111,0b01110001,
                   0b11011101,0b11000000,0b00010111,0b01110111,0b00011101,0b00010101,0b01011100,0b01110111,
                   0b01110111,0b01000110,0b10111010,0b00111010,0b10111000,0b00000000};   

// letzten Teilblock ggf. mit Nullen auffuellen; Anzahl der Bloecke zaehlen und Wert zwischen eckigen Klammern eintragen
// gefundenen Wert um 1 erhoehen und als Endwert in Codezeile 146 einsetzen

////////////////////////////////////////////////////////
void writePLL(uint32_t PLLword) {          // this routine clocks a 32 bits word into the ADF4351
                                           // msb (b31) first, lsb (b0) last 
  for (byte flop=32; flop>0; flop--) {     // PLL word has 32 bits 
    (PLLword & 0x80000000? PORTB |= 0b00001000 : PORTB &= 0b11110111);   // AND with MSB to clock in '1' or '0' on PB3 = 'pin11'                                                                             
    PORTB |= 0b00010000;                   // clock in bit on rising edge of CLK (PB4 = 'pin12' = 1)
    PORTB &= 0b11101111;                   // CLK (PB4 = 0)      
    PLLword <<= 1;                         // rotate left for next bit
} // for flop 
    PORTB |= 0b00000100;                   // latch in PLL word on rising edge of LE (PB2 = 'pin10' = 1)
    PORTB &= 0b11111011;                   // LE (PB2 = 0)      
} // writePLL

////////////////////////////////////////////////////////////////////////////
void setup () {   
Serial.begin(9600);
Serial.println("Start");
pinMode(2, INPUT_PULLUP);
pinMode(3, INPUT_PULLUP);
pinMode(4, INPUT_PULLUP);
pinMode(13, OUTPUT);

DDRB  = 0xff; // PB7-0 are all outputs 
PORTB = 0x00; // make PB7-0 low

 // select channel 1-4 via #3 and #4
 //-------------------------------------------- 
 // Ch.1: Fout= 144.975 MHz 
 if(digitalRead(3)==HIGH&&digitalRead(4)==HIGH)
 { 
   r0 = 0x2E0310;  
   r1 = 0x80083E9;
   r2 = 0x4E42; 
   r3 = 0x4B3;
   r4 = 0xCC81FC;
   r5 = 0x580005; 
 }
 
 // Ch.2: Fout= 432.975 MHz
 else if(digitalRead(3)==LOW&&digitalRead(4)==HIGH)
 { 
   r0 = 0x450228;  
   r1 = 0x80083E9;
   r2 = 0x4E42; 
   r3 = 0x4B3;
   r4 = 0xBC803C;
   r5 = 0x580005; 
 }
 
 // Ch.3: Fout= 1296.975 MHz 
 else if(digitalRead(3)==HIGH&&digitalRead(4)==LOW)
  { 
    r0 = 0x338BD8;  
    r1 = 0x8008FA1;
    r2 = 0x4E42; 
    r3 = 0x4B3;
    r4 = 0x9C81FC;
    r5 = 0x580005; 
  }

 // Ch.4: Fout= 2320.975 MHz 
 else if(digitalRead(3)==LOW&&digitalRead(4)==LOW) 
   { 
     r0 = 0x2E1A38; 
     r1 = 0x8009F41;
     r2 = 0x4E42; 
     r3 = 0x4B3;
     r4 = 0x8C81FC;
     r5 = 0x580005; 
   }
     
     writePLL(r5); 
     writePLL(r4); 
     writePLL(r3); 
     writePLL(r2); 
     writePLL(r1); 
     writePLL(r0);  
   }

/////////////////////////////////////////////////////////////////
void loop() {                              // main loop     
    if(digitalRead(2)==HIGH)
    {
    Serial.println("CW-Start");
    delay(750);                            // pause before transmitting beacon text
    for (int i=0; i<23; i++) {             // message array contains 22 bytes, see definition above
    digitalWrite(13, HIGH);
                                           // if i = uint8_t then code is 8 bytes larger (?!, try it ;-)                             
    byte flop = message[i];                // isolate pending message byte         
    for (uint8_t y=8; y>0; y--) {          // pending message byte has ... yes! ... 8 bits ;-)         
    writePLL(flop & 0x80? r4 : (r4-32));   // key transmitter (ADF4351 Reg4, b5) depending on msb value of flop 
    delay(cwtick);                         // delay cwtick            
    digitalWrite(13, LOW);
          flop <<= 1;                      // rotate left for next (most significant) bit            
     } // for y         
  } // for i     
      delay(350);                          // after beacon text small pause
      writePLL(r4);                        // TX on    
      Serial.println("CW-Stop");
    }
    else
    { writePLL(r4);}                        // TX on    
      delay(15000);                         // for 15 seconds
    }





